/* AbiWord
 * Copyright (C) 2001 AbiSource, Inc.
 * Copyright (C) 2002-2003 Marc Maurer (j.m.maurer@student.utwente.nl)
 * Copyright (C) 2001-2003 William Lachance (wlach@interlog.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

/* See bug 1764
 * This product is not manufactured, approved, or supported by 
 * Corel Corporation or Corel Corporation Limited."
 */

#ifndef IE_IMP_WP_H
#define IE_IMP_WP_H

#include <stdio.h>
#include <libwpd/libwpd.h>
#include "ie_imp.h"
#include "ut_string.h"
#include "ut_string_class.h"
#include "ut_growbuf.h"
#include "ut_mbtowc.h"
#include "pd_Document.h"
#include "fl_AutoNum.h"

// The importer/reader for WordPerfect 6 documents.

#define WP_WORDPERFECT6789_EXPECTED_MAJOR_VERSION 2
#define WP_WORDPERFECT_DOCUMENT_FILE_TYPE 10

#define WP_HEADER_PRODUCT_TYPE_OFFSET 8
#define WP_HEADER_FILE_TYPE_OFFSET 9
#define WP_HEADER_MAJOR_VERSION_OFFSET 10
#define WP_HEADER_MINOR_VERSION_OFFSET 11

#define WP_NUM_LIST_LEVELS 8

// These are the defaults for generating list text
#define WP_PARAGRAPH_STYLE_MARGIN_LEFT_INCREMENT 0.5f // I have no idea if this is correct, but it seems to look reasonably good
#define WP_PARAGRAPH_STYLE_TEXT_INDENT_DECREMENT -0.3f // I have no idea if this is correct, but it seems to look reasonably good


class ABI_EXPORT IE_Imp_WordPerfect_Sniffer : public IE_ImpSniffer
{
    friend class IE_Imp;
    friend class IE_Imp_WordPerfect;
    
public:
    IE_Imp_WordPerfect_Sniffer();
    virtual ~IE_Imp_WordPerfect_Sniffer();
    
    virtual UT_Confidence_t recognizeContents (const char * szBuf,
					       UT_uint32 iNumbytes);
    virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
    virtual bool getDlgLabels (const char ** szDesc,
			       const char ** szSuffixList,
			       IEFileType * ft);
    virtual UT_Error constructImporter (PD_Document * pDocument,
					IE_Imp ** ppie);
};

class ABI_EXPORT IE_Imp_WordPerfect : public IE_Imp, public WPXHLListenerImpl
{
public:
    IE_Imp_WordPerfect(PD_Document * pDocument);
    ~IE_Imp_WordPerfect();
    
    virtual UT_Error importFile(const char * szFilename);
    virtual void pasteFromBuffer(PD_DocumentRange * pDocRange,
				 UT_uint8 * pData, UT_uint32 lenData, const char * szEncoding = 0);
    
    virtual void startDocument();
    virtual void endDocument();
    virtual void openParagraph(const guint8 paragraphJustification, const guint32 textAttributeBits,
			       const gchar *fontName, const gfloat fontSize, 
			       const gboolean isColumnBreak, const gboolean isPageBreak);
    virtual void openSpan(const guint32 textAttributeBits, const gchar *fontName, const gfloat fontSize);
    virtual void openSection(guint numColumns, gfloat marginLeft, gfloat marginRight);
    virtual void insertText(const guint16 *textArray, const guint len);
    virtual void insertLineBreak();
    
    virtual void openTable();
    virtual void openRow();
    virtual void openCell(const guint32 col, const guint32 row, const guint32 colSpan, const guint32 rowSpan, const RGBSColor * cellFgColor, const RGBSColor * cellBgColor);
    virtual void closeTable();
    
protected:
    UT_Error							_appendSpan(const guint32 textAttributeBits, const gchar *fontName, const gfloat fontSize);
    UT_Error							_appendParagraph(const guint8 paragraphJustification, const guint32 textAttributeBits,
										 const gchar *fontName, const gfloat fontSize);
    UT_Error							_appendSection(guint numColumns, gfloat marginLeft, gfloat marginRight);
    
    // section props
    bool								m_bLeftMarginSet;
    bool								m_bRightMarginSet;
    float								m_leftMargin;
    float								m_rightMargin;
    bool								m_bColumnsSet;
    UT_uint8							m_numColumns;
    
    // state handling that libwpd can't account for
    bool								m_bParagraphChanged;
    bool								m_bParagraphInSection;
    bool								m_bInSection;
    bool								m_bSectionChanged;
    
private:
    bool								m_bInCell;
};
#endif /* IE_IMP_WP_H */
